import { Platform, PixelRatio } from 'react-native';

function capsize(options) {
  if ('leading' in options && 'lineGap' in options) {
    throw new Error('Only a single line height style can be provided. Please pass either `lineGap` OR `leading`.');
  }

  if ('capHeight' in options && 'fontSize' in options) {
    throw new Error('Please pass either `capHeight` OR `fontSize`, not both.');
  }

  const {
    fontMetrics
  } = options;
  const capHeightScale = fontMetrics.capHeight / fontMetrics.unitsPerEm;
  let specifiedFontSize;
  let specifiedCapHeight;

  if ('capHeight' in options) {
    specifiedFontSize = options.capHeight / capHeightScale;
    specifiedCapHeight = options.capHeight;
  } else if ('fontSize' in options) {
    specifiedFontSize = options.fontSize;
    specifiedCapHeight = options.fontSize * capHeightScale;
  } else {
    throw new Error('Please pass either `capHeight` OR `fontSize`.');
  }

  let specifiedLineHeight;

  if ('lineGap' in options) {
    specifiedLineHeight = specifiedCapHeight + options.lineGap;
  } else if ('leading' in options) {
    specifiedLineHeight = options.leading;
  }

  return createStyle({
    lineHeight: specifiedLineHeight,
    fontSize: specifiedFontSize,
    fontMetrics
  });
}

function createStyle({
  lineHeight,
  fontSize,
  fontMetrics
}) {
  const toScale = value => value / fontSize;

  const absoluteDescent = Math.abs(fontMetrics.descent);
  const capHeightScale = fontMetrics.capHeight / fontMetrics.unitsPerEm;
  const descentScale = absoluteDescent / fontMetrics.unitsPerEm;
  const ascentScale = fontMetrics.ascent / fontMetrics.unitsPerEm;
  const lineGapScale = fontMetrics.lineGap / fontMetrics.unitsPerEm;
  const contentArea = fontMetrics.ascent + fontMetrics.lineGap + absoluteDescent;
  const lineHeightScale = contentArea / fontMetrics.unitsPerEm;
  const lineHeightNormal = lineHeightScale * fontSize;
  const specifiedLineHeightOffset = lineHeight ? (lineHeightNormal - lineHeight) / 2 : 0;

  const leadingTrim = value => value - toScale(specifiedLineHeightOffset);

  const fontScale = PixelRatio.getFontScale();
  let style = {
    fontSize,
    marginTop: leadingTrim(ascentScale - capHeightScale + lineGapScale / 2) * -fontSize * fontScale,
    marginBottom: Platform.OS === 'web' ? leadingTrim(descentScale + lineGapScale / 2) * -fontSize : (leadingTrim(descentScale + lineGapScale / 2) * -fontSize - Math.max(0, specifiedLineHeightOffset)) * fontScale,
    paddingTop: Platform.OS === 'web' ? 0 : Math.max(0, specifiedLineHeightOffset) * fontScale
  };

  if (lineHeight) {
    style = { ...style,
      lineHeight: lineHeight
    };
  }

  return style;
}

export default capsize;
export const getCapHeight = ({
  fontSize,
  fontMetrics
}) => fontSize * fontMetrics.capHeight / fontMetrics.unitsPerEm;
//# sourceMappingURL=index.js.map